/* *INDENT-OFF* */
!/* PSPP - computes sample statistics.
!   Copyright (C) 1997, 1998 Free Software Foundation, Inc.
!   Written by Ben Pfaff <blp@gnu.org>.
!
!   This program is free software; you can redistribute it and/or
!   modify it under the terms of the GNU General Public License as
!   published by the Free Software Foundation; either version 2 of the
!   License, or (at your option) any later version.
!
!   This program is distributed in the hope that it will be useful, but
!   WITHOUT ANY WARRANTY; without even the implied warranty of
!   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
!   General Public License for more details.
!
!   You should have received a copy of the GNU General Public License
!   along with this program; if not, write to the Free Software
!   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
!   02111-1307, USA. */
!
!/*
! FILE HANDLE command.
!
! This command does not accept any standard format at the moment.  This
! ought to change in the future.
!
! entry points:
!   cmd_file_handle()
!   get_handle_by_name()
!   get_handle_by_filename()
!*/

$ "FILE HANDLE":
$   name=string;
$   recform=recform:fixed/!variable/spanned;
$   lrecl=integer;
$   mode=mode:!character/image/binary/multipunch/_360.
/* *INDENT-ON* */

#include <config.h>
#include <assert.h>
#include <errno.h>
#include <stdlib.h>
#include "common.h"
#include "avl.h"
#include "error.h"
#include "filename.h"
#include "getline.h"
#include "var.h"
#include "file-handle.h"

/* (header) */
/* (decls global) */

#undef DEBUGGING
/*#define DEBUGGING 1*/
#include "debug-print.h"

avl_tree *files;
file_handle *inline_file;

static int internal_cmd_file_handle (void);
static void init_file_handle (file_handle * handle);

#if 0
/* Prepends the current directory (the one with the current syntax
   file) to the filename, and returns a malloc()'d copy of the
   resulting name. */
char *
prepend_current_directory (const char *filename)
{
  if (!is_special_filename (filename))
    {
      char *current_dir = getl_get_current_directory ();
      char *filename_with_dir = prepend_dir (filename, current_dir);

      free (current_dir);
      return filename_with_dir;
    }
  else
    return xstrdup (filename);
}
#endif

int
cmd_file_handle (void)
{
  int code;
  code = internal_cmd_file_handle ();
  if (s_name)
    free (s_name);
  return code;
}

static int
internal_cmd_file_handle (void)
{
  char handle_name[9];
  char *handle_name_p = handle_name;
  file_handle *fp;

  get_token ();
  force_id ();
  strcpy (handle_name, tokstr);

  fp = NULL;
  if (files)
    fp = avl_find (files, &handle_name_p);
  if (fp)
    return msg (SE, _("File handle %s had already been defined to refer to "
		"file %s.  It is not possible to redefine a file "
		"handle within a session."), tokstr, fp->fn);

  get_token ();
  force_match ('/');

  /* (parser bare) */

  if (token != '.')
    return syntax_error (_("expecting end of command"));

  if (!s_name)
    return msg (SE, _("The FILE HANDLE required subcommand NAME "
		"is not present."));

  fp = xmalloc (sizeof (file_handle));
  init_file_handle (fp);

  switch (recform)
    {
    case FIXED:
      if (n_lrecl == NOT_LONG)
	{
	  msg (SE, _("Fixed length records were specified on /RECFORM, but "
	       "record length was not specified on /LRECL.  80-character "
	       "records will be assumed."));
	  n_lrecl = 80;
	}
      else if (n_lrecl < 1)
	{
	  msg (SE, _("Record length (%ld) must be at least one byte.  "
	       "80-character records will be assumed."), n_lrecl);
	  n_lrecl = 80;
	}
      fp->recform = FH_RF_FIXED;
      fp->lrecl = n_lrecl;
      break;
    case VARIABLE:
      fp->recform = FH_RF_VARIABLE;
      break;
    case SPANNED:
      msg (SE, _("/RECFORM SPANNED is not implemented, as the author doesn't "
	   "know what it is supposed to do.  Send the author a note."));
      break;
    default:
      assert (0);
    }

  switch (mode)
    {
    case CHARACTER:
      fp->mode = FH_MD_CHARACTER;
      break;
    case IMAGE:
      msg (SE, _("/MODE IMAGE is not implemented, as the author doesn't know "
	   "what it is supposed to do.  Send the author a note."));
      break;
    case BINARY:
      fp->mode = FH_MD_BINARY;
      break;
    case MULTIPUNCH:
      msg (SE, _("/MODE MULTIPUNCH is not implemented.  If you care, "
		 "complain."));
      break;
    case _360:
      msg (SE, _("/MODE 360 is not implemented.  If you care, complain."));
      break;
    default:
      assert (0);
    }

  fp->name = xstrdup (handle_name);
  {
#if 0
    char *filename_with_dir = prepend_current_directory (s_name);
    fp->norm_fn = normalize_filename (filename_with_dir);
    free (filename_with_dir);
#else
    fp->norm_fn = normalize_filename (s_name);
#endif
  }
  fp->fn = s_name;
  s_name = NULL;
  avl_force_insert (files, fp);

  return 1;
}

/* File handle functions. */

/* Sets up some fields in H; caller should fill in
   H->{NAME,NORM_FN,FN}. */
static void
init_file_handle (file_handle * h)
{
  h->recform = FH_RF_VARIABLE;
  h->mode = FH_MD_CHARACTER;
  h->ext = NULL;
  h->class = NULL;
}

/* Returns the handle corresponding to FILENAME.  Creates the handle
   if no handle exists for that file.  All filenames are normalized
   first, so different filenames referring to the same file will
   return the same file handle. */
file_handle *
fh_get_handle_by_filename (const char *filename)
{
  file_handle f, *fp;
  char *fn;
  char *name;
  int len;

  /* Get filename. */
  {
#if 0
    char *filename_with_dir = prepend_current_directory (filename);
    fn = normalize_filename (filename_with_dir);
    free (filename_with_dir);
#else
    fn = normalize_filename (filename);
#endif
  }
  len = strlen (fn);

  /* Create handle name with invalid identifier character to prevent
     conflicts with handles created with FILE HANDLE. */
  name = xmalloc (len + 2);
  name[0] = '*';
  strcpy (&name[1], fn);

  f.name = name;
  fp = avl_find (files, &f);
  if (!fp)
    {
      fp = xmalloc (sizeof (file_handle));
      init_file_handle (fp);
      fp->name = name;
      fp->norm_fn = fn;
      fp->fn = xstrdup (filename);
      avl_force_insert (files, fp);
    }
  else
    {
      free (fn);
      free (name);
    }
  return fp;
}

/* Returns the handle with identifier NAME, if it exists; otherwise
   reports error to user and returns NULL. */
file_handle *
fh_get_handle_by_name (const char name[9])
{
  file_handle f, *fp;
  f.name = (char *) name;
  fp = avl_find (files, &f);

  if (!fp)
    msg (SE, _("File handle `%s' has not been previously declared on "
	 "FILE HANDLE."), name);
  return fp;
}

/* Returns the identifier of file HANDLE.  If HANDLE was created by
   referring to a filename (i.e., DATA LIST FILE='yyy' instead of FILE
   HANDLE XXX='yyy'), returns the filename, enclosed in double quotes.
   Return value is in a static buffer.

   Useful for printing error messages about use of file handles.  */
const char *
fh_handle_name (file_handle *h)
{
  static char *buf = NULL;

  if (buf)
    {
      free (buf);
      buf = NULL;
    }
  if (!h)
    return NULL;

  if (h->name[0] == '*')
    {
      int len = strlen (h->fn);

      buf = xmalloc (len + 3);
      strcpy (&buf[1], h->fn);
      buf[0] = buf[len + 1] = '"';
      buf[len + 2] = 0;
      return buf;
    }
  return h->name;
}

/* Closes the stdio FILE associated with handle H.  Frees internal
   buffers associated with that file.  Does *not* destroy the file
   handle H.  (File handles are permanent during a session.)  */
void
fh_close_handle (file_handle *h)
{
  if (h == NULL)
    return;

  debug_printf (("Closing %s%s.\n", fh_handle_name (h),
		 h->class == NULL ? " (already closed)" : ""));

  if (h->class)
    h->class->close (h);
  h->class = NULL;
  h->ext = NULL;
}

/* Compares names of file handles A and B. */
static int
cmp_file_handle (const void *a, const void *b, unused void *foo)
{
  return strcmp (((file_handle *) a)->name, ((file_handle *) b)->name);
}

/* Initialize the AVL tree of file handles; inserts the "inline file"
   inline_file. */
void
fh_init_files (void)
{
  /* Create AVL tree. */
  files = avl_create (NULL, cmp_file_handle, NULL);

  /* Insert inline file. */
  inline_file = xmalloc (sizeof (file_handle));
  init_file_handle (inline_file);
  inline_file->name = "INLINE";
  inline_file->fn = inline_file->norm_fn = (char *) _("<Inline File>");
  avl_force_insert (files, inline_file);
}

/* Parses a file handle name, which may be a filename as a string or
   a file handle name as an identifier.  Returns the file handle or
   NULL on failure. */
file_handle *
fh_parse_file_handle (void)
{
  file_handle *handle;

  if (token == ID)
    handle = fh_get_handle_by_name (tokstr);
  else if (token == STRING)
    handle = fh_get_handle_by_filename (tokstr);
  else
    {
      msg (SE, _("A file name or handle was expected on the FILE subcommand."));
      return NULL;
    }

  if (!handle)
    return NULL;
  get_token ();

  return handle;
}

/* Returns the (normalized) filename associated with file handle H. */
char *
fh_handle_filename (file_handle * h)
{
  return h->norm_fn;
}

/* Returns the width of a logical record on file handle H. */
size_t
fh_record_width (file_handle *h)
{
  if (h == inline_file)
    return 80;
  else if (h->recform == FH_RF_FIXED)
    return h->lrecl;
  else
    return 1024;
}

/* Local Variables: */
/* mode:c */
/* End: */
